<?php

namespace FiloBlu\Cegid\Controller\Seller;

use FiloBlu\Cegid\Api\Data\SellerQuoteInterface;
use FiloBlu\Cegid\Api\Data\SellerQuoteInterfaceFactory;
use FiloBlu\Cegid\Api\SellerQuoteRepositoryInterface;
use Magento\Checkout\Model\Session;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\Result\Json;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\Exception\NotFoundException;
use Magento\Store\Model\ScopeInterface;
use Zend_Db;

/**
 * Class Link
 * @package FiloBlu\Cegid\Controller\Seller
 */
class Link extends Action
{
    /**
     * @var string
     */
    const XML_PATH_ENABLED = 'filoblu_cegid/seller/enable';

    /**
     * @var JsonFactory
     */
    protected $resultJsonFactory;

    /**
     * @var ResourceConnection
     */
    protected $resourceConnection;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var Session
     */
    private $checkoutSession;
    /**
     * @var SellerQuoteInterfaceFactory
     */
    private $sellerQuoteFactory;
    /**
     * @var SellerQuoteRepositoryInterface
     */
    private $sellerQuoteRepository;
    protected $quoteIdMaskFactory;

    /**
     * Search constructor.
     * @param Context $context
     * @param JsonFactory $resultJsonFactory
     * @param Session $checkoutSession
     * @param SellerQuoteInterfaceFactory $sellerQuoteFactory
     * @param SellerQuoteRepositoryInterface $sellerQuoteRepository
     * @param ResourceConnection $resourceConnection
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(Context $context,
                                JsonFactory $resultJsonFactory,
                                Session $checkoutSession,
                                SellerQuoteInterfaceFactory $sellerQuoteFactory,
                                SellerQuoteRepositoryInterface $sellerQuoteRepository,
                                ResourceConnection $resourceConnection,
                                ScopeConfigInterface $scopeConfig
    )
    {
        $this->resultJsonFactory = $resultJsonFactory;
        $this->resourceConnection = $resourceConnection;
        $this->scopeConfig = $scopeConfig;
        $this->checkoutSession = $checkoutSession;
        $this->sellerQuoteFactory = $sellerQuoteFactory;
        $this->sellerQuoteRepository = $sellerQuoteRepository;

        parent::__construct($context);
    }

    /**
     * @return ResponseInterface|Json|ResultInterface
     */
    public function execute()
    {
        $post = $this->getRequest()->getPost();


       $this->quoteIdMaskFactory =  ObjectManager::getInstance()->get(\Magento\Quote\Model\QuoteIdMaskFactory::class);
        $quoteIdMask = $this->quoteIdMaskFactory->create()->load($post['quote'], 'masked_id');
        $currentQuoteId = $quoteIdMask->getQuoteId();
       // $currentQuoteId = $this->checkoutSession->getQuoteId();
        /** @var SellerQuoteInterface $sellerQuoteLink */
        $sellerQuoteLink = $this->sellerQuoteFactory->create();
        $sellerQuoteLink->setQuote($currentQuoteId)->setSeller($post['cegid']['seller']['id']);
        $this->sellerQuoteRepository->save($sellerQuoteLink);

        return $this->resultJsonFactory
            ->create()
            ->setData([]);

    }

    /**
     * Dispatch request
     *
     * @param RequestInterface $request
     * @return ResponseInterface
     * @throws NotFoundException
     */
    public function dispatch(RequestInterface $request)
    {
        if ($this->scopeConfig->isSetFlag(self::XML_PATH_ENABLED, ScopeInterface::SCOPE_STORE)) {
            return parent::dispatch($request);
        }

        throw new NotFoundException(__('Page not found.'));
    }
}