<?php

namespace FiloBlu\Cegid\Helper\Cegid;

// Confirmed (polling)
use FiloBlu\CegidY2\Api\SaleDocumentService\ServiceType\Get;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\GetByReferenceFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\GetByReference_Request;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\SaleDocumentReference;

// Update
use FiloBlu\CegidY2\Api\SaleDocumentService\ServiceType\Update;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\UpdateHeaderFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\UpdateHeader_Request;

use FiloBlu\Cegid\Api\SellerRepositoryInterface;
use FiloBlu\CegidY2\Api\SaleDocumentService\ArrayType\ArrayOfCreate_Line;
use FiloBlu\CegidY2\Api\SaleDocumentService\ArrayType\ArrayOfCreate_LineFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\EnumType\DocumentOrigin;
use FiloBlu\CegidY2\Api\SaleDocumentService\EnumType\SaleDocumentType;
use FiloBlu\CegidY2\Api\SaleDocumentService\ServiceType\Create;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create_Header;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create_HeaderFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create_LineFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create_Request;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create_RequestFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\CreateFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\CreateResponse;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\ItemIdentifierFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\RetailContext;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\RetailContextFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\AddressFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\OmniChannelLine;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\OmniChannelLineFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create_Tax;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\SaleDocumentIdentifier;
use Magento\Framework\Api\SortOrder;

use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create_Payment;
use FiloBlu\CegidY2\Api\SaleDocumentService\ArrayType\ArrayOfCreate_Payment;

use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\OmniChannel;

use FiloBlu\CegidY2\Service\CredentialProvider;
use FiloBlu\CegidY2\Service\ServiceProvider;
use FiloBlu\CegidY2\Api\SaleDocumentService\ArrayType\ArrayOfCreate_TaxFactory;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Customer\Api\Data\CustomerInterface;
use Magento\Customer\Api\Data\CustomerInterfaceFactory;
use Magento\Framework\Api\SearchCriteriaBuilderFactory;
use Magento\Framework\Exception\LocalizedException as LocalizedExceptionAlias;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\InventorySalesApi\Api\Data\SalesChannelInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Quote\Api\CartRepositoryInterface;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Store\Model\StoreManagerInterface;
use RuntimeException;

use Magento\Directory\Model\ResourceModel\Country\CollectionFactory;

use FiloBlu\Cegid\Helper\Cegid\Base as BaseHelper;
use FiloBlu\ExtInventory\Helper\Sources as ExtInvHelperSources;
use FiloBlu\ExtInventory\Helper\Data as ExtInvHelperData;
use FiloBlu\ExtInventory\Model\ShippingAlgorithms\SourceSlots\SourcesData as SourceSlotsSourcesData;

use Magento\Rma\Api\RmaRepositoryInterface;
use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Api\FilterFactory;
use Magento\Framework\Api\Search\FilterGroupBuilderFactory;
use Magento\Framework\Api\Search\FilterGroupFactory;
use Magento\Framework\App\ResourceConnection;

/**
 * Class Order
 * @package FiloBlu\Cegid\Helper\Cegid
 */
class Order extends BaseHelper
{
    const EUR_CODE = 'EUR';

    /**
     * @var CredentialProvider
     */
    protected $credentialProvider;
    /**
     * @var RetailContextFactory
     */
    protected $retailContextFactory;
    /**
     * @var ServiceProvider
     */
    protected $createService;
    /**
     * @var CreateFactory
     */
    protected $createFactory;
    /**
     * @var Create_RequestFactory
     */
    protected $createRequestFactory;
    /**
     * @var CustomerRepositoryInterface
     */
    protected $customerRepository;
    /**
     * @var Create_HeaderFactory
     */
    protected $createHeaderFactory;
    /**
     * @var ArrayOfCreate_LineFactory
     */
    protected $arrayOfCreateLineFactory;
    /**
     * @var Create_LineFactory
     */
    protected $createLineFactory;
    /**
     * @var CartRepositoryInterface
     */
    protected $quoteRepository;
    /**
     * @var Customer
     */
    protected $customerHelper;
    /**
     * @var ItemIdentifierFactory
     */
    protected $itemIdentifierFactory;
    /**
     * @var CustomerInterfaceFactory
     */
    protected $customerFactory;
    /**
     * @var SellerRepositoryInterface
     */
    protected $sellerRepository;
    /**
     * @var SearchCriteriaBuilderFactory
     */
    protected $searchCriteriaBuilderFactory;
    /**
     * @var AddressFactory
     */
    protected $addressFactory;
    /**
     * @var BaseHelper
     */
    protected $baseHelper;
    /**
     * @var Create_Payment
     */
    protected $createPayment;
    /**
     * @var ArrayOfCreate_Payment
     */
    protected $arrayOfCreatePayment;
    /**
     * @var CollectionFactory
     */
    protected $countryCollectionFactory;
    /**
     * @var OmniChannel
     */
    protected $omniChannel;
    /**
     * @var ExtInvHelperSources
     */
    public $extInvHelperSources;
    /**
     * @var ExtInvHelperData
     */
    public $extInvHelperData;
    /**
     * @var SourceSlotsSourcesData
     */
    public $sourceSlotsSourcesData;
    /**
     * @var OrderRepositoryInterface
     */
    public $orderRepositoryInterface;
    /**
     * @var ServiceProvider
     */
    protected $getService;
    /**
     * @var GetByReferenceFactory
     */
    protected $getByReferenceFactory;
    /**
     * @var ServiceProvider
     */
    protected $updateService;
    /**
     * @var UpdateHeaderFactory
     */
    protected $updateHeaderFactory;
    /**
     * @var ResourceConnection
     */
    protected $resourceConnection;
    /**
     * @var \Magento\Framework\DB\Adapter\AdapterInterface
     */
    protected $connection;

    protected $omniChannelLine;
    protected $omniChannelLineFactory;
    protected $rmaRepositoryInterface;
    protected $filterFactory;
    protected $filterGroupFactory;
    /**
     * @var ArrayOfCreate_TaxFactory
     */
    protected $arrayOfCreate_TaxFactory;

    protected $create_Tax;

    protected $loadedCustomerCegidId;

    // Init variables
    protected $order_source_items_quantities;
    protected $is_groupage_order;
    protected $is_main_source_order;
    protected $is_source_order;
    protected $cegid_customer_id;
    protected $stock_id;
    protected $is_return = false;
    // Temp variables
    protected $current_source_code;
    protected $current_array_quantities;
    protected $current_order_to_cegid_grandtotal = 0;
    protected $current_order;

    public function __construct(CredentialProvider $credentialProvider,
                                CreateFactory $createFactory,
                                CustomerRepositoryInterface $customerRepository,
                                Create_HeaderFactory $createHeaderFactory,
                                RetailContextFactory $retailContextFactory,
                                Create_RequestFactory $createRequestFactory,
                                Create_LineFactory $createLineFactory,
                                ArrayOfCreate_LineFactory $arrayOfCreateLineFactory,
                                ItemIdentifierFactory $itemIdentifierFactory,
                                CartRepositoryInterface $quoteRepository,
                                ServiceProvider $createService,
                                CustomerInterfaceFactory $customerFactory,
                                SearchCriteriaBuilderFactory $searchCriteriaBuilderFactory,
                                SellerRepositoryInterface $sellerRepository,
                                Customer $customerHelper,
                                AddressFactory $addressFactory,
                                BaseHelper $baseHelper,
                                Create_Payment $createPayment,
                                ArrayOfCreate_Payment $arrayOfCreatePayment,
                                CollectionFactory $countryCollectionFactory,
                                OmniChannel $omniChannel,
                                \Magento\Framework\App\Helper\Context $context,
                                ExtInvHelperSources $extInvHelperSources,
                                ExtInvHelperData $extInvHelperData,
                                StoreManagerInterface $storeManager,
                                SourceSlotsSourcesData $sourceSlotsSourcesData,
                                OmniChannelLine $omniChannelLine,
                                OmniChannelLineFactory $omniChannelLineFactory,
                                RmaRepositoryInterface $rmaRepositoryInterface,
                                SearchCriteriaInterface $searchCriteriaInterface,
                                FilterFactory $filterFactory,
                                FilterGroupFactory $filterGroupFactory,
                                ArrayOfCreate_TaxFactory $arrayOfCreate_TaxFactory,
                                Create_Tax $create_Tax,
                                OrderRepositoryInterface $orderRepositoryInterface,
                                ServiceProvider $getService,
                                GetByReferenceFactory $getByReferenceFactory,
                                UpdateHeaderFactory $updateHeaderFactory,
                                ServiceProvider $updateService,
                                ResourceConnection $resourceConnection
    )
    {
        parent::__construct($context);
        $this->credentialProvider = $credentialProvider;
        $this->createFactory = $createFactory;
        $this->retailContextFactory = $retailContextFactory;
        $this->createService = $createService;
        $this->createRequestFactory = $createRequestFactory;
        $this->customerRepository = $customerRepository;
        $this->arrayOfCreateLineFactory = $arrayOfCreateLineFactory;
        $this->createHeaderFactory = $createHeaderFactory;
        $this->createLineFactory = $createLineFactory;
        $this->quoteRepository = $quoteRepository;
        $this->customerHelper = $customerHelper;
        $this->itemIdentifierFactory = $itemIdentifierFactory;
        $this->customerFactory = $customerFactory;
        $this->sellerRepository = $sellerRepository;
        $this->searchCriteriaBuilderFactory = $searchCriteriaBuilderFactory;
        $this->addressFactory = $addressFactory;
        $this->baseHelper = $baseHelper;
        $this->createPayment = $createPayment;
        $this->arrayOfCreatePayment = $arrayOfCreatePayment;
        $this->countryCollectionFactory = $countryCollectionFactory;
        $this->omniChannel = $omniChannel;
        $this->extInvHelperSources = $extInvHelperSources;
        $this->extInvHelperData = $extInvHelperData;
        $this->storeManager = $storeManager;
        $this->sourceSlotsSourcesData = $sourceSlotsSourcesData;
        $this->omniChannelLine = $omniChannelLine;
        $this->omniChannelLineFactory = $omniChannelLineFactory;
        $this->rmaRepositoryInterface = $rmaRepositoryInterface;
        $this->searchCriteriaInterface = $searchCriteriaInterface;
        $this->filterFactory = $filterFactory;
        $this->filterGroupFactory = $filterGroupFactory;
        $this->arrayOfCreate_TaxFactory = $arrayOfCreate_TaxFactory;
        $this->create_Tax = $create_Tax;
        $this->orderRepositoryInterface = $orderRepositoryInterface;
        $this->getService = $getService;
        $this->getByReferenceFactory = $getByReferenceFactory;
        $this->updateHeaderFactory = $updateHeaderFactory;
        $this->updateService = $updateService;
        $this->resourceConnection = $resourceConnection;
    }

    /*
     * Some useful variables
     */
    public function orderCegidInit(OrderInterface $order){
        // Load all FiloBlu Order Items (table inventory_fb_orderitem_source)
        // [source code] => array(sku => qty)
        $this->order_source_items_quantities = $this->getOrderSourceItemsQuantities($order);
        $this->is_groupage_order = $this->isGroupageOrder($order);
        $this->is_main_source_order = $this->isMainSourceOrder($order);
        $this->is_source_order = $this->isSourceOrder($order);
        $this->cegid_customer_id = $this->getCustomerCegidFromOrder($order);
        $this->stock_id = $this->extInvHelperData->getStockIdByOrder($order);
    }

    /**
     * @param OrderInterface $order
     * We need to call CEGID for each warehouse ID
     */
    public function createOrders(OrderInterface $order){
        $this->orderCegidInit($order);
        $errors = array();
        foreach ($this->order_source_items_quantities as $source_code => $array_quantities) {
            $this->current_source_code = $source_code;
            $this->current_array_quantities = $array_quantities;
            $result = $this->createOrder($order);
            $this->current_order_to_cegid_grandtotal = 0;

            if ($result !== true){
                $errors[] = $result;
            }
        }

        if (!empty($errors)){
            throw new RuntimeException(implode("\n", $errors));
        }
        return true;
    }

    /**
     * @param OrderInterface $order
     * @return array
     * Return array with all the sources and for each source the skus have barcode and quantity involved
     */
    function getOrderSourceItemsQuantities(OrderInterface $order){
        $list_of_involved_sources = array();
        foreach ($order->getItems() as $item) {
            if($item->getProductType() == 'simple'){
                $data = $item->getProduct()->getData();
                $filobluOrderItems = $item->getExtensionAttributes()->getFilobluOrderItem();
                if(is_array($filobluOrderItems) && count($filobluOrderItems)>0){
                    foreach ($filobluOrderItems as $filobluOrderItem) {
                        $list_of_involved_sources[$filobluOrderItem->getInventorySourceId()][$filobluOrderItem->getSku()]['qty'] = $filobluOrderItem->getQty();
                        $list_of_involved_sources[$filobluOrderItem->getInventorySourceId()][$filobluOrderItem->getSku()]['barcode'] = $data['barcode'];
                    }
                }
            }
        }
        return $list_of_involved_sources;
    }


    protected function getOrderShippingAndBillingAddresses(OrderInterface $order){
        if ($isGroupage = $this->isGroupageOrder($order)){
            $main_source = $this->extInvHelperSources->getSource($this->stock_id,$this->sourceSlotsSourcesData->getMainSourceCode());
            $shipping_address = $billing_address = $this->extInvHelperSources->getSourceAddress($main_source);
        } else {
            $shipping_address = $order->getShippingAddress();
            $billing_address = $order->getBillingAddress();
        }

        return array('shipping_address' => $shipping_address,'billing_address' => $billing_address);
    }


    /**
     * @param OrderInterface $order
     * @return array|bool
     * @throws LocalizedExceptionAlias
     * @throws NoSuchEntityException
     * @throws \Magento\Framework\Exception\InputException
     * @throws \Magento\Framework\Exception\State\InputMismatchException
     */
    public function createOrder(OrderInterface $order)
    {
        $addresses = $this->getOrderShippingAndBillingAddresses($order);
        $shipping_address = $addresses['shipping_address'];
        $billing_address = $addresses['billing_address'];

        $header = $this->getHeader($order);
        $lines = $this->getLines($order);
        /** @var Create_Request $request */
        $request = $this->createRequestFactory->create([
            'header' => $header
        ]);

        $request->setLines($lines);

        $deliveryAddress = $this->getGeneratedAddress($shipping_address);
        $request->setDeliveryAddress($deliveryAddress);

        $invoicingAddress = $this->getGeneratedAddress($billing_address);
        $request->setInvoicingAddress($invoicingAddress);

        $payments = $this->getPayments($order);
        $request->setPayments($payments);

        $parameters = $this->createFactory->create();
        $parameters->setCreateRequest($request);
        $parameters->setClientContext($this->getRetailContext());

        // Execute
        $createService = $this->createService->create();
        $createService->Create($parameters);
        // Save data
        $request = $createService->getLastRequest();
        $response = $createService->getLastResponse();
        $error = array_values($createService->getLastError());
        $result = $createService->getResult();

        $this->writeLog('createOrder',$order,$request,$response);

        if(!$result && $error && count($error)){
            return $error[0]->getMessage();
        }
        return true;

        //throw new RuntimeException(implode("\n", $createService->getLastError()));
    }

    /**
     * @param OrderInterface $order
     * @param bool $is_return
     * @return Create_Header
     * @throws LocalizedExceptionAlias
     * @throws NoSuchEntityException
     * @throws \Magento\Framework\Exception\InputException
     * @throws \Magento\Framework\Exception\State\InputMismatchException
     */
    public function getHeader(OrderInterface $order)
    {
        /** @var Create_Header $header */
        $header = $this->createHeaderFactory->create();

        $header->setTaxExcluded(true);
        $header->setCurrencyId(self::EUR_CODE);
        $header->setCustomerId($this->cegid_customer_id);
        $header->setInternalReference($this->getOrderReference($order,$this->current_source_code));
        $header->setLinesUnmodifiable(true);
        $header->setExternalReference($this->getOrderReference($order,$this->current_source_code));
        $header->setDate(substr($order->getUpdatedAt(), 0, 10));
        $header->setActive(true);
        //$header->setWarehouseId($this->sourceSlotsSourcesData->getThirdPartyCodeFromSourceCode($this->current_source_code));

        $omnichannel = $this->getOmniChannel($order);
        $header->setOmniChannel($omnichannel);
        //TaxExcluded deve andare a true perchè UnitPrice e NetUnitPrice sono IVA ESCLUSA
        $header->setTaxExcluded(true);
        $type = $this->getConfigSaleDocumentType();
        $header->setType($type);
        $header->setOrigin($this->getConfigSaleDocumentOrigin());
        $store_id = $this->getConfigCegidStoreId();
        $header->setStoreId($store_id);

        return $header;
    }

    /**
     * @param OrderInterface $order
     * @param $origin
     * @param array $return_items
     * @return ArrayOfCreate_Line
     */
    public function getLines(OrderInterface $order, $return_items = false)
    {
        $lines = $this->arrayOfCreateLineFactory->create();
        $origin = $this->getConfigSaleDocumentOrigin();

        foreach ($order->getItems() as $item) {

            if($item->getProductType() != 'configurable'){
                continue;
            }

            //if!in_array($item->getSku(), array_keys($this->order_source_items_quantities[$this->current_source_code])
            // This call should have only the items of the current source_code i'm reading
            if(!array_key_exists($item->getSku(), $this->order_source_items_quantities[$this->current_source_code])){
                continue;
            }
            // I need to split the grand total for each source, and everything must be in EUR
            // This MUST BE with VAT
            $single_unit_row_total = ($item->getBaseRowTotalInclTax() - $item->getBaseDiscountAmount()) / $item->getQtyOrdered();
            if ($order->getBaseCurrencyCode() !== self::EUR_CODE){
                $single_unit_row_total *= $order->getBaseToGlobalRate();
            }

            $line = $this->createLineFactory->create();
            $barcode = $this->order_source_items_quantities[$this->current_source_code][$item->getSku()]['barcode'];
            $qty_to_take = $this->order_source_items_quantities[$this->current_source_code][$item->getSku()]['qty'];

            $this_order_row_total = $single_unit_row_total * $qty_to_take;
            $this->current_order_to_cegid_grandtotal += $this_order_row_total;

            $identifier = $this->itemIdentifierFactory->create(['reference' => $barcode]);
            $line->setItemIdentifier($identifier);
            $line->setOrigin($origin);
            $line->setQuantity($qty_to_take); //Not $item->getQtyOrdered()
            // Importo Iva a default articolo. In questo caso non va passata l’intera sezione Taxes
            //$line->setTaxes($this->getTaxes($item)); // This must be before the unit price

            // Importo unitario (UnitPrice) e Importo Netto Sconti (Net Unit Price) IVA ESCLUSA.
            $net_unit_price = $item->getBaseRowTotal() / $item->getQtyOrdered();
            $unit_price = ($item->getBaseRowTotal() - $item->getBaseDiscountAmount()) / $item->getQtyOrdered();
            if ($order->getBaseCurrencyCode() !== self::EUR_CODE){
                $net_unit_price *= $order->getBaseToGlobalRate();
                $unit_price *= $order->getBaseToGlobalRate();
            }
            $line->setNetUnitPrice($net_unit_price);
            // prezzo IVA ESCLUSA SENZA SCONTI  di una unità
            $line->setUnitPrice($unit_price);
            if($this->current_source_code == $this->sourceSlotsSourcesData->getMainSourceCode()){
                // Do not set OmniChannel on line if this is not a "from source" order
            } else {
                //$omniChannelLine = $this->getOmniChannelLine($this->current_source_code);
                $omniChannelLine = $this->getCegidOrderOmniChannelLine($this->current_source_code);
                $line->setOmniChannel($omniChannelLine);
            }

            $lines->addToCreate_Line($line);

        }

        return $lines;
    }

    public function getCegidOrderOmniChannelLine($current_source_code){
        $third_party_code = $this->sourceSlotsSourcesData->getThirdPartyCodeFromSourceCode($current_source_code);
        $omniChannelLine = new OmniChannelLine();
        $omniChannelLine->setWarehouseId($third_party_code);
        return $omniChannelLine;
    }

    public function getTaxes($item){
        $arrayOfCreate_Tax = $this->arrayOfCreate_TaxFactory->create();
        $arrayOfCreate_Tax->addToCreate_Tax($this->getCreateTax($item));
        return $arrayOfCreate_Tax;
    }

    public function getCreateTax($item){
        $create_Tax = new Create_Tax();
        $create_Tax->setAmount($item->getTaxAmount());
        //$create_Tax->setFamilyId("NOR"); //Without FamilyId we can send custom VAT percentage (will be calculated)
        return $create_Tax;
    }

    public function getPayments(OrderInterface $order)
    {
        $payments = new ArrayOfCreate_Payment();

        $payment = new Create_Payment();

        // Importo del pagamento (Amount sotto la sezione Payments) deve riportare il totale IVA INCLUSA.
        //$gtotal = $order->getBaseGrandTotal();
        //if ($order->getBaseCurrencyCode() !== self::EUR_CODE){
        //    $gtotal *= $order->getBaseToGlobalRate();
        //}

        // If I have a multistock order then i will send ONE order for EACH source
        // each order must have his how splitted grand total
        $gtotal = $this->current_order_to_cegid_grandtotal;

        $payment->setAmount($gtotal);
        $payment->setCurrencyId(self::EUR_CODE);
        //Sempre a false is received payment
        $payment->setIsReceivedPayment(false);
        // Can't avoid duplicates because the ID must be an integer
        //$payment->setId($order->getPayment()->getEntityId()."-".$this->sourceSlotsSourcesData->getThirdPartyCodeFromSourceCode($this->current_source_code));
        $payment->setId($order->getPayment()->getEntityId());
        // TODO: Method ID fisso o mappatura?
        //$payment->setMethodId($order->getPayment()->getMethod());
        $payment->setMethodId($this->getCegidPaymentMethodId($order));

        $date = new \DateTime($order->getCreatedAt());
        $payment->setDueDate($date->format('Y-m-d'));

        $payments->addToCreate_Payment($payment);

        return $payments;
    }

    public function getOmniChannel(OrderInterface $order){
        $omnichannel = new OmniChannel();
        $omnichannel->setBillingStatus('Pending');
        $omnichannel->setDeliveryType('ShipByCentral');

        if($this->current_source_code == $this->sourceSlotsSourcesData->getMainSourceCode()){
            $omnichannel->setFollowUpStatus('Validated');
        } else {
            $omnichannel->setFollowUpStatus('WaitingCommodity');
        }

        $omnichannel->setGiftMessage('None');
        $omnichannel->setGiftMessageType('None');
        // TODO: Method ID fisso o mappatura?
        //$omnichannel->setPaymentMethodId($this->getCegidPaymentMethodId($order));
        //$omnichannel->setPaymentMethodId($order->getPayment()->getMethod());

        // E' altrettanto importante, specialmente per gli ordini con stock negozio,
        // che il PaymentStatus sia a "Totally" invece che "Debited".
        // E' prevista in questo caso una chiamata di conferma del pagamento
        // (passandolo quindi a "Debited") una volta che il negozio avrà
        // confermato la disponibilità del prodotto bloccandola.
        if($this->current_source_code == $this->sourceSlotsSourcesData->getMainSourceCode() && !$this->is_groupage_order){
            $omnichannel->setPaymentStatus('Debited');
        } else {
            $omnichannel->setPaymentStatus('Totally');
        }
        $omnichannel->setReturnStatus('NotReturned');
        $omnichannel->setShippingStatus('Pending');
        return $omnichannel;
    }

    /**
     * @param $order_address
     * @return \FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Address
     */
    public function getGeneratedAddress($order_address){

        $iso3CountryCode = $this->countryCollectionFactory->create()
            ->addFieldToFilter('country_id', $order_address->getData('country_id'))
            ->getFirstItem()
            ->getData('iso3_code');

        $region_code = ($order_address->getRegionCode() ? $order_address->getRegionCode() : $order_address->getData('region_code'));

        $address = $this->addressFactory->create();
        $address->setCity($order_address->getData('city'));
        $address->setCountryId($iso3CountryCode);
        $address->setFirstName($order_address->getData('firstname'));
        $address->setLastName($order_address->getData('lastname'));
        $address->setLine1($this->baseHelper->getAddressStreet($order_address,1));
        $address->setLine2($this->baseHelper->getAddressStreet($order_address,2));
        $address->setLine3($this->baseHelper->getAddressStreet($order_address,3));
        $address->setPhoneNumber($order_address->getData('telephone'));
        $address->setRegion($region_code);
        $address->setZipCode($order_address->getData('postcode'));
        return $address;
    }

    /**
     * @return RetailContext
     */
    public function getRetailContext()
    {
        return $this->retailContextFactory->create(['databaseId' =>
            $this->credentialProvider->getDatabase('sale_document_service')
        ]);
    }

    public function getConfigSaleDocumentType(){
        $return = $this->scopeConfig->getValue('filoblu_cegid_y2/sale_document_service/sale_document_type', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        if(!$return){
            return SaleDocumentType::VALUE_CUSTOMER_ORDER;
        }
        return $return;
    }

    public function getConfigSaleDocumentOrigin(){
        $return =  $this->scopeConfig->getValue('filoblu_cegid_y2/sale_document_service/sale_document_origin', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        if(!$return){
            return DocumentOrigin::VALUE_ECOMMERCE;
        }
        return $return;
    }

    public function getConfigCegidStoreId(){
        $return = $this->scopeConfig->getValue('filoblu_cegid_y2/sale_document_service/cegid_store_id', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        if(!$return){
            return '000';
        }
        return (string)$return;
    }

    /**
     * @param OrderInterface $order
     * @return bool
     * Ordine Misto
     */
    public function isGroupageOrder(OrderInterface $order){
        try {
            $sources = $this->getOrderSources($order);
            if(count($sources) > 1){
                return true;
            }
        } catch (\Exception $e) {}
        return false;
    }

    /**
     * @param OrderInterface $order
     * @return bool
     * Ordine Negozio
     */
    public function isSourceOrder(OrderInterface $order){
        try {
            $sources = $this->getOrderSources($order);
            if(count($sources) > 1){
                return false;
            }
            if (count($sources) === 1 && $sources[0] !== $this->sourceSlotsSourcesData->getMainSourceCode()){
                return true;
            }
        } catch (\Exception $e) {}
        return false;
    }

    /**
     * @param OrderInterface $order
     * @return bool
     * Ordine Main Source (SNATT)
     */
    public function isMainSourceOrder(OrderInterface $order) {
        if (!$this->isGroupageOrder($order) && !$this->isSourceOrder($order)){
            return true;
        }
        return false;
    }

    /**
     * @param OrderInterface $order
     * @return array
     */
    public function getOrderSources(OrderInterface $order){
        $sources = array();
        foreach ($order->getItems() as $item) {
            if($item->getProductType() !== 'simple') {continue;}

            $filoblu_order_items = $item->getExtensionAttributes()->getFilobluOrderItem();
            if (count($filoblu_order_items) > 0){
                foreach ($filoblu_order_items as $filoblu_order_item) {
                    $sources[] = $filoblu_order_item->getInventorySourceId();
                }
            }
        }
        $sources = array_values(array_filter(array_unique($sources)));
        return $sources;
    }

    // Return how many pieces to get from which sources
    // Ex. Item ABC x2 -> x1 from source Alpha & x1 from source Beta
    public function getFilobluOrderSourceItems($item){
        $result = array();
        $filoblu_order_items = $item->getExtensionAttributes()->getFilobluOrderItem();
        if (count($filoblu_order_items) > 0){
            foreach ($filoblu_order_items as $filoblu_order_item) {
                $result[$filoblu_order_item->getInventorySourceId()] = $filoblu_order_item->getQty();
            }
        }
        return $result;
    }

    public function getCustomerCegidFromOrder(OrderInterface $order){

        if ($this->loadedCustomerCegidId){
            return $this->loadedCustomerCegidId;
        }

        $customerId = $order->getCustomerId();

        if ($customerId === null || (int)$order->getCustomerIsGuest() === 1) {
            /* GUEST */
            $email = $order->getCustomerEmail();
            $customer = $this->customerHelper->customerInterfaceFactoryCreate();
            $customer->setEmail($email);

            $response = $this->customerHelper->searchCustomerByEmail($customer);

            if (!$response || $response === null || $response->getSearchCustomerIdsResult()->getCustomerQueryData() === null) {
                $shippingAddress = $order->getShippingAddress();

                $customer->setLastname($shippingAddress->getLastName());
                $customer->setFirstname($shippingAddress->getFirstname());

                $customer = $this->customerHelper->createCustomer($customer);
                $customerCegidId = $this->getCompatibleCustomerCustomAttribute($customer,'cegid_id');
            } else {
                $customerCegidId = $response->getSearchCustomerIdsResult()->getCustomerQueryData()[0]->getCustomerId();
            }

        } else {

            $customer = $this->customerRepository->getById($order->getCustomerId());
            $customerCegidId = $this->getCompatibleCustomerCustomAttribute($customer,'cegid_id');
            if (!$customerCegidId) {
                $response = $this->customerHelper->searchCustomerByEmail($customer);

                if (!$response || $response === null || $response->getSearchCustomerIdsResult()->getCustomerQueryData() == null) {
                    $customer->setLastname($customer->getLastname());
                    $customer->setFirstname($customer->getFirstname());
                    $customer = $this->customerHelper->createCustomer($customer);
                    $customerCegidId = $this->getCompatibleCustomerCustomAttribute($customer,'cegid_id');

                } else {
                    $customerCegidId = $response->getSearchCustomerIdsResult()->getCustomerQueryData()[0]->getCustomerId();
                }
            }
        }

        $this->loadedCustomerCegidId = $customerCegidId;

        return $customerCegidId;
    }

    public function getOrderReference(OrderInterface $order,$source_code = null){
        if (!$source_code){
            return $order->getIncrementId();
        }
        $third_party_code = $this->sourceSlotsSourcesData->getThirdPartyCodeFromSourceCode($source_code);
        return strtolower("{$order->getIncrementId()}-{$third_party_code}");
    }

    /**
     * @param OrderInterface $order
     * @return bool
     * Return true if the goods are confirmed by the source
     * Return false if the goods are denied by the source
     */
    public function checkOrderConfirmed(OrderInterface $order){
        $this->orderCegidInit($order);
        foreach ($this->order_source_items_quantities as $source_code => $array_quantities) {
            $this->current_source_code = $source_code;
            $this->current_array_quantities = $array_quantities;
            $confirmed = $this->getByReference($order);
            if(!$confirmed){
                return false;
            }
        }
        return true;
    }

    protected function getByReference(OrderInterface $order){

        $orderReference = $this->getOrderReference($order,$this->current_source_code);

        $saleDocumentReference = new SaleDocumentReference(
            $this->cegid_customer_id,
            $orderReference,
            SaleDocumentType::VALUE_CUSTOMER_ORDER
        );
        $getByReference_Request = new GetByReference_Request($saleDocumentReference);

        $parameters = $this->getByReferenceFactory->create();
        $parameters->setSearchRequest($getByReference_Request);
        $parameters->setClientContext($this->getRetailContext());

        // Execute
        $getService = $this->getService->create();
        $getService->GetByReference($parameters);
        // Save data
        $request = $getService->getLastRequest();
        $response = $getService->getLastResponse();
        $error = array_values($getService->getLastError());
        $result = $getService->getResult();

        $this->writeLog(
            'getByReference polling per conferma prenotazione merce da parte del negozio',
            $order,
            $request,
            $response
        );

        if(!$result){
            return false;
        }

        $followUpStatus = $getService->getResult()->GetByReferenceResult->Header->OmniChannel->FollowUpStatus;
        if($followUpStatus == 'Validated'){
            return true;
        }
        if ($followUpStatus == 'BookingRefused'){
            return false;
        }

        if($error && count($error)){
            throw new RuntimeException($error[0]->getMessage());
        }
        throw new RuntimeException('Unknown error');

    }

    /*
     * SECTION FOR UPDATE
     */
    public function sendPaymentConfirmations(OrderInterface $order){
        $this->orderCegidInit($order);
        $errors = array();
        foreach ($this->order_source_items_quantities as $source_code => $array_quantities) {
            $this->current_source_code = $source_code;
            $this->current_array_quantities = $array_quantities;
            $result = $this->sendPaymentConfirmation($order);
            if ($result !== true){
                $errors[] = $result;
            }
        }
        if (!empty($errors)){
            throw new RuntimeException(implode("\n", $errors));
        }
        return true;
    }

    public function sendPaymentConfirmation($order){

        $orderReference = $this->getOrderReference($order,$this->current_source_code);

        $addresses = $this->getOrderShippingAndBillingAddresses($order);
        $shipping_address = $addresses['shipping_address'];
        $billing_address = $addresses['billing_address'];
        $deliveryAddress = $this->getGeneratedAddress($shipping_address);
        $invoicingAddress = $this->getGeneratedAddress($billing_address);

        $saleDocumentReference = new SaleDocumentReference(
            $this->cegid_customer_id,
            $orderReference,
            SaleDocumentType::VALUE_CUSTOMER_ORDER
        );

        $identifier = new SaleDocumentIdentifier();
        $identifier->setReference($saleDocumentReference);

        $omniChannel = $this->getOmniChannel($order);
        $omniChannel->setFollowUpStatus('Validated');
        $omniChannel->setPaymentStatus('Debited');
        $omniChannel->setShippingStatus('Pending');
        $omniChannel->setTracking('TrackingNumberTest123');
        $omniChannel->setTransporter('TransporterNameTest123');

        $updateHeaderRequest = new UpdateHeader_Request();
        $updateHeaderRequest->setInvoicingAddress($invoicingAddress);
        $updateHeaderRequest->setDeliveryAddress($deliveryAddress);
        $updateHeaderRequest->setIdentifier($identifier);
        $updateHeaderRequest->setOmniChannel($omniChannel);

        $parameters = $this->updateHeaderFactory->create();
        $parameters->setUpdateHeaderRequest($updateHeaderRequest);
        $parameters->setClientContext($this->getRetailContext());

        // Execute
        $updateService = $this->updateService->create();
        $updateService->UpdateHeader($parameters);
        // Save data
        $request = $updateService->getLastRequest();
        $response = $updateService->getLastResponse();
        $error = array_values($updateService->getLastError());
        $result = $updateService->getResult();

        $this->writeLog(
            'sendPaymentConfirmation invio conferma di pagamento a CEGID dopo avvenuto capture',
            $order,
            $request,
            $response
        );

        if(!$result && $error && count($error)){
            return $error[0]->getMessage();
        }
        return true;
    }

    /**
     * @return \Magento\Framework\DB\Adapter\AdapterInterface
     */
    protected function getConnection()
    {
        if (!$this->connection) {
            $this->connection = $this->resourceConnection->getConnection('core_read');
        }
        return $this->connection;
    }



}